import React, { Fragment } from 'react';
import * as PropTypes from 'prop-types';
import DateFilter from './filterComponents/DateTimeFilter';

// import DateFilter from './filterComponents/DateFilter';
import TextFilter from './filterComponents/TextFilter';
import BooleanFilter from './filterComponents/BooleanFilter';
import NumberFilter from './filterComponents/NumberFilter';

class FilterComponent extends React.Component {
    constructor(props) {
        super(props);
        const textBoxes = {};
        const selectedFilterOption = {};

        // Default values for filter text and or filter options
        props.columnList.map((col) => {
            if (col.type === 'string' && col.format !== 'date') {
                textBoxes[col.field] = '';
                selectedFilterOption[col.field] = 'contains';
            }
            else if (col.type === 'object') {
                textBoxes[col.field] = '';
                selectedFilterOption[col.field] = 'contains';
            }
            else if (col.type === 'boolean') {
                textBoxes[col.field] = '';
                selectedFilterOption[col.field] = '';
            }
            else if (col.type === 'number' || col.type === 'integer') {
                textBoxes[col.field] = {
                    num: '',
                };
                selectedFilterOption[col.field] = 'equals';
            }
            else if (col.type === 'date' || col.format === 'date') {
                textBoxes[col.field] = {
                    date: null,
                    date2: null,
                };
                selectedFilterOption[col.field] = 'equals';
            }
        });

        const { left } = props;

        this.state = {
            // Filter text/value for columns
            textBoxes: textBoxes,

            // Selected filter options, e.g. equals, contains etc for columns
            selectedFilterOption: selectedFilterOption,

            // ScrollLeft value for filter component
            left: left,

            // left: 0,
        };
    }

    componentWillReceiveProps(nextProps) {
        this.setState({ left: nextProps.left });
    }

    render() {
        const {
            textBoxes,
            selectedFilterOption,
        } = this.state;

        const {
            isFilterShowing,
            columnList,
            colWidths,
            contextStyles,
        } = this.props;

        const reOrderedColList = this.reOrderColList(columnList);

        // Inline styles to make the component simpler
        const dateFilterStyles = {
            display: 'inline-block',
            alignItems: 'center',
            margin: '8px',
        };

        const nameStyle = {
            margin: '8px',
            display: 'inline-block',
        };

        // Styles for buttons container
        const buttonContainerStyles = {
            display: 'flex',
        };

        // Styles for continue button
        const continueButtonStyles = {
            display: 'block',
            margin: '8px',
            position: 'sticky',
            left: '8px',
        };

        // Styles for reset button
        const resetButtonStyles = {
            display: 'block',
            margin: '8px',
            position: 'sticky',
            left: '83px',
        };

        // Styles for cancel button
        const cancelButtonStyles = {
            display: 'block',
            margin: '8px',
            position: 'sticky',
            left: '140px',
        };

        const buttons = [
            {
                text: 'Continue',
                styles: { ...continueButtonStyles },
                clickHandler: this.handleApplyButtonClick,
            },
            {
                text: 'Reset',
                styles: { ...resetButtonStyles },
                clickHandler: this.handleResetButtonClick,
            },
            {
                text: 'Cancel',
                styles: { ...cancelButtonStyles },
                clickHandler: this.handleCancelButtonClick,
            },
        ];

        // Styles for input
        const selectStyles = {
            marginLeft: '0px',
            marginBottom: '2px',
            display: 'block',
            width: '100%',
        };

        // Styles for input
        const inputStyles = {
            width: 'inherit',
        };

        // Styles for left scroll button
        const leftScrollButtonStyles = {
            position: 'fixed',
            left: '-15px',
            top: '56px',
            height: '30px',
            width: '30px',
            backgroundColor: 'aquamarine',
            borderRadius: '15px',
            color: 'red',
        };

        // Styles for right scroll button
        const rightScrollButtonStyles = {
            position: 'fixed',
            right: '11px',
            top: '56px',
            height: '30px',
            width: '15px',
            backgroundColor: 'aquamarine',
            borderBottomLeftRadius: '15px',
            borderTopLeftRadius: '15px',
            color: 'red',
        };

        return (
            isFilterShowing ?
                <Fragment>
                    <div
                        role="presentation"
                        style={ leftScrollButtonStyles }
                        onClick={ this.handleScrollLeft }
                        ref={ (node) => this.filterDialogNode = node }
                        id="scrollLeft">
                        <div
                            style={ {
                                position: 'absolute',
                                left: '17px',
                                top: '9px',
                                backgroundColor: 'aquamarine',
                                color: 'red',
                            } }>
                            { '<' }
                        </div>
                    </div>
                    <div
                        role="presentation"
                        style={ rightScrollButtonStyles }
                        onClick={ this.handleScrollRight }
                        id="scrollRight">
                        <div
                            style={ {
                                position: 'absolute',
                                left: '5px',
                                top: '9px',
                                backgroundColor: 'aquamarine',
                                color: 'red',
                            } }>
                            { '>' }
                        </div>
                    </div>
                    <div
                        style={ contextStyles.filter }
                        className="combinedFilter">
                        <div
                            style={ {
                                display: 'inline-flex',
                            } }
                            className="columnFilters">
                            {
                                reOrderedColList.map(( col ) => {
                                    if (col.type === 'date' || col.format === 'date' || col.format === 'time' || col.format === 'date-time') {
                                        return (
                                            <DateFilter
                                                key={ `${col.field}__filter` }
                                                field={ col.field }
                                                format={ col.format }
                                                placeholder={ col.headerName }
                                                width={ `${colWidths.get(col.field) - 16}px` }
                                                textBoxes={ textBoxes }
                                                selectedFilterOption={ selectedFilterOption }
                                                inputStyles={ inputStyles }
                                                selectStyles={ selectStyles }
                                                filterStyle={ dateFilterStyles }
                                                onDateChanged={ this.handleFilterChange } />
                                        );
                                    }
                                    else if (col.type === 'string') {
                                        return (
                                            <TextFilter
                                                key={ `${col.field}__filter` }
                                                field={ col.field }
                                                placeholder={ col.headerName }
                                                width={ `${colWidths.get(col.field) - 16}px` }
                                                textBoxes={ textBoxes }
                                                selectedFilterOption={ selectedFilterOption }
                                                inputStyles={ inputStyles }
                                                selectStyles={ selectStyles }
                                                onTextChanged={ this.handleFilterChange }
                                                nameStyle={ nameStyle } />
                                        );
                                    }
                                    else if ((col.type === 'object')) {
                                        return (
                                            <TextFilter
                                                key={ `${col.field}__filter` }
                                                field={ col.field }
                                                placeholder={ col.headerName }
                                                width={ `${colWidths.get(col.field) - 16}px` }
                                                textBoxes={ textBoxes }
                                                selectedFilterOption={ selectedFilterOption }
                                                inputStyles={ inputStyles }
                                                selectStyles={ selectStyles }
                                                onTextChanged={ this.handleFilterChange }
                                                nameStyle={ nameStyle } />
                                        );
                                    }
                                    else if (col.type === 'boolean') {
                                        return (
                                            <BooleanFilter
                                                key={ `${col.field}__filter` }
                                                field={ col.field }
                                                placeholder={ col.headerName }
                                                width={ `${colWidths.get(col.field) - 16}px` }
                                                textBoxes={ textBoxes }
                                                selectedFilterOption={ selectedFilterOption }
                                                inputStyles={ inputStyles }
                                                selectStyles={ selectStyles }
                                                onValueChanged={ this.handleFilterChange }
                                                nameStyle={ nameStyle } />
                                        );
                                    }
                                    else if (col.type === 'number' || col.type === 'integer') {
                                        return (
                                            <NumberFilter
                                                key={ `${col.field}__filter` }
                                                field={ col.field }
                                                placeholder={ col.headerName }
                                                width={ `${colWidths.get(col.field) - 16}px` }
                                                textBoxes={ textBoxes }
                                                selectedFilterOption={ selectedFilterOption }
                                                inputStyles={ inputStyles }
                                                selectStyles={ selectStyles }
                                                filterStyle={ dateFilterStyles }
                                                onNumChanged={ this.handleFilterChange } />
                                        );
                                    }
                                    else {
                                        return (
                                            <TextFilter
                                                key={ `${col.field}__filter` }
                                                field={ col.field }
                                                placeholder={ col.headerName }
                                                width={ `${colWidths.get(col.field) - 16}px` }
                                                textBoxes={ textBoxes }
                                                selectedFilterOption={ selectedFilterOption }
                                                inputStyles={ inputStyles }
                                                selectStyles={ selectStyles }
                                                onTextChanged={ this.handleFilterChange }
                                                nameStyle={ nameStyle } />
                                        );
                                    }
                                })
                            }
                        </div>
                        <div style={ buttonContainerStyles }>
                            {
                                buttons.map((button) => (
                                    <button
                                        style={ button.styles }
                                        key={ `${button.text}__button` }
                                        onClick={ button.clickHandler }
                                        type="button">
                                        { button.text }
                                    </button>
                                ))
                            }
                        </div>
                    </div>
                </Fragment>
                :
                null
        );
    }

    componentDidUpdate() {
        const filterContainer = document.getElementById('filter');
        const { left } = this.state;
        filterContainer.scrollLeft = left;
    }

    componentDidMount() {
        document.addEventListener('mousedown', this.handleClick, false);
    }

    componentWillUnmount() {
        document.removeEventListener('mousedown', this.handleClick, false);
    }

    /**
     * Handles click on grid to be able to hide filter dialog on click outside the filter
     */
    handleClick = (e) => {
        const agBodyViewport = document.getElementsByClassName('ag-body-viewport')[0];
        const filterContainer = document.getElementById('filter');
        const { onFilterToggle } = this.props;

        // Check for click inside body viewport but outside of filterContainer.
        if (agBodyViewport && agBodyViewport.contains(e.target) && !filterContainer.contains(e.target)) {
            const { isFilterShowing } = this.props;

            if (isFilterShowing) {
                onFilterToggle(false);
            }
        }
        else {
            return;
        }
    }

    /**
     * Reorders column filters in same order of cols, i.e. pinned cols first followed by unpinned cols.
     * @param {Array} cols column list
     * @returns reordered list
     */
    reOrderColList = (cols) => {
        const newList = [];

        // Inserting pinned cols.
        cols.map((col) => {
            if (col.pinned) {
                newList.push(col);
            }
        });

        // Inserting unpinned cols.
        cols.map((col) => {
            if (!col.pinned) {
                newList.push(col);
            }
        });
        return newList;
    };

    /** No. of pixels scrolled at one click. */
    scrollStepSize = 100;

    /**
     * Scroll to left.
     * @method handleScrollLeft
     */
    handleScrollLeft = () => {
        const filterContainer = document.getElementById('filter');
        const left = filterContainer.scrollLeft;

        if (left > this.scrollStepSize) {
            filterContainer.scrollLeft = left - this.scrollStepSize;
        }
        else {
            filterContainer.scrollLeft = 0;
        }
    }

    /**
     * Scroll to right.
     * @method handleScrollRight
     */
    handleScrollRight = () => {
        const filterContainer = document.getElementById('filter');
        const left = filterContainer.scrollLeft;

        if (left + this.scrollStepSize < filterContainer.scrollWidth - filterContainer.clientWidth) {
            filterContainer.scrollLeft = left + this.scrollStepSize;
        }
        else {
            filterContainer.scrollLeft = filterContainer.scrollWidth - filterContainer.clientWidth;
        }
    }

    /** Reset all fiter inputs to default value */
    handleResetButtonClick = () => {
        const { columnList } = this.props;
        const textBoxes = {};
        const selectedFilterOption = {};
        columnList.map((col) => {
            if (col.type === 'date' || col.format === 'date') {
                textBoxes[col.field] = {
                    date: null,
                    date2: null,
                };
                selectedFilterOption[col.field] = 'equals';
            }
            else if ((col.type === 'string')) {
                textBoxes[col.field] = '';
                selectedFilterOption[col.field] = 'contains';
            }
            else if (col.type === 'number' || col.type === 'integer') {
                textBoxes[col.field] = {
                    num: '',
                    num2: '',
                };
                selectedFilterOption[col.field] = 'equals';
            }
            else if (col.type === 'boolean') {
                textBoxes[col.field] = '';
                selectedFilterOption[col.field] = '';
            }
        });
        this.setState({
            textBoxes,
            selectedFilterOption,
        });
    }

    /** Compose filterData object and pass on, on click of apply/continue. */
    handleApplyButtonClick = () => {
        const {
            textBoxes,
            selectedFilterOption,
        } = this.state;
        this.setState({
            textBoxes,
        }, () => {
            const {
                onFilterToggle,
                onApplyFilters,
                columnList,
            } = this.props;

            const filterData = {};

            columnList.map((col) => {
                filterData[col.field] = {
                    value: textBoxes[col.field],
                    option: selectedFilterOption[col.field],
                };
            });

            onApplyFilters(filterData)
                .then(() => {
                    onFilterToggle(false);
                });
        });
    }

    /** Hides filter dialog. */
    handleCancelButtonClick = () => {
        const { onFilterToggle } = this.props;
        if (onFilterToggle) {
            onFilterToggle(false);
        }
    }

    /**
     * Handles changes in filter components for each column
     * @param {String} field Identifier for column
     * @param {Object} data Filter text/data corresponding to the fields/ columns
     * @param {String} option Filter option, e.g. equals, contains, greater than etc.
     */
    handleFilterChange = (field, data, option) => {
        const {
            textBoxes,
            selectedFilterOption,
        } = this.state;
        this.setState({
            textBoxes: {
                ...textBoxes,
                ...data,
            },
            selectedFilterOption: {
                ...selectedFilterOption,
                [field]: option,
            },
        });
    }
}

FilterComponent.propTypes = {
    /** Pass on filter data (text and option corresponding to columns)
     * @param {object} filterData Object containing columns as key and corresponding filter text and filter option in an object as value.
     */
    onApplyFilters: PropTypes.func,

    /** Style object */
    contextStyles: PropTypes.object,

    /** Array containing details on columns */
    columnList: PropTypes.arrayOf(PropTypes.object),

    /** If the filter is showing. */
    isFilterShowing: PropTypes.bool,

    /** Mapping of column and their width */
    colWidths: PropTypes.instanceOf(Map),

    /** Scroll position */
    left: PropTypes.number,

    /**
     * Method to toggle show/hide of filter.
     * @param {boolean} value true or false to show or hide filter dialog
     */
    onFilterToggle: PropTypes.func,
};

export default FilterComponent;