import React, { Component } from 'react';
import PropTypes from 'prop-types';
import MUIMenu from '@material-ui/core/Menu';

const ITEM_HEIGHT = 48;

class Menu extends Component {
    static propTypes = {
        children: PropTypes.arrayOf(PropTypes.node).isRequired,
        button: PropTypes.node.isRequired,
    };

    constructor(props) {
        super(props);
        this.state = {
            anchorEl: null,
        };
    }

    render() {
        const {
            button,
            children,
        } = this.props;
        const {
            anchorEl,
        } = this.state;

        // Inject the ability to close the menu on item click
        const kids = React.Children.map(children, (child) => {
            const {
                children: itemChildren,
                onClick,
            } = child.props;
            return React.cloneElement(
                child,
                {
                    onClick: (...args) => {
                        if (onClick) {
                            onClick(...args);
                        }
                        this.handleClose();
                    },
                },
                itemChildren,
            );
        });

        // Inject the ability to open the menu on button click
        const {
            children: buttonChildren,
            onClick: buttonClick,
        } = button.props;
        const Trigger = React.cloneElement(
            button,
            {
                onClick: (...args) => {
                    this.handleClick(...args);
                    if (buttonClick) {
                        buttonClick(...args);
                    }
                },
            },
            buttonChildren,
        );

        return (
            <div>
                { Trigger }
                <MUIMenu
                    id="long-menu"
                    anchorEl={ anchorEl }
                    keepMounted
                    open={ !!anchorEl }
                    onClose={ this.handleClose }
                    PaperProps={ {
                        style: {
                            maxHeight: ITEM_HEIGHT * 4.5,
                        },
                    } }>
                    { kids }
                </MUIMenu>
            </div>
        );
    }

    handleClick = (event) => {
        this.setState({
            anchorEl: event.currentTarget,
        });
    };

    handleClose = () => {
        this.setState({
            anchorEl: null,
        });
    };

    show(event) {
        if (event && event.currentTarget) {
            this.setState({
                anchorEl: event.currentTarget,
            });
        }
    };
}

export default Menu;
